<?php

/*
 * This file is part of SwiftMailer.
 * (c) 2004-2009 Chris Corbyn
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/*
 * List of MIME type automatically detected in Swift Mailer.
 */

//You may add or take away what you like (lowercase required)

$swift_mime_types = array(
    'aif'  => 'audio/x-aiff',
    'aiff' => 'audio/x-aiff',
    'avi'  => 'video/avi',
    'bmp'  => 'image/bmp',
    'bz2'  => 'application/x-bz2',
    'csv'  => 'text/csv',
    'dmg'  => 'application/x-apple-diskimage',
    'doc'  => 'application/msword',
    'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'eml'  => 'message/rfc822',
    'aps'  => 'application/postscript',
    'exe'  => 'application/x-ms-dos-executable',
    'flv'  => 'video/x-flv',
    'gif'  => 'image/gif',
    'gz'   => 'application/x-gzip',
    'hqx'  => 'application/stuffit',
    'htm'  => 'text/html',
    'html' => 'text/html',
    'jar'  => 'application/x-java-archive',
    'jpeg' => 'image/jpeg',
    'jpg'  => 'image/jpeg',
    'm3u'  => 'audio/x-mpegurl',
    'm4a'  => 'audio/mp4',
    'mdb'  => 'application/x-msaccess',
    'mid'  => 'audio/midi',
    'midi' => 'audio/midi',
    'mov'  => 'video/quicktime',
    'mp3'  => 'audio/mpeg',
    'mp4'  => 'video/mp4',
    'mpeg' => 'video/mpeg',
    'mpg'  => 'video/mpeg',
    'odg'  => 'vnd.oasis.opendocument.graphics',
    'odp'  => 'vnd.oasis.opendocument.presentation',
    'odt'  => 'vnd.oasis.opendocument.text',
    'ods'  => 'vnd.oasis.opendocument.spreadsheet',
    'ogg'  => 'audio/ogg',
    'pdf'  => 'application/pdf',
    'png'  => 'image/png',
    'ppt'  => 'application/vnd.ms-powerpoint',
    'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'ps'   => 'application/postscript',
    'rar'  => 'application/x-rar-compressed',
    'rtf'  => 'application/rtf',
    'tar'  => 'application/x-tar',
    'sit'  => 'application/x-stuffit',
    'svg'  => 'image/svg+xml',
    'tif'  => 'image/tiff',
    'tiff' => 'image/tiff',
    'ttf'  => 'application/x-font-truetype',
    'txt'  => 'text/plain',
    'vcf'  => 'text/x-vcard',
    'wav'  => 'audio/wav',
    'wma'  => 'audio/x-ms-wma',
    'wmv'  => 'audio/x-ms-wmv',
    'xls'  => 'application/excel',
    'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'xml'  => 'application/xml',
    'zip'  => 'application/zip'
);
